/**
 * Copyright © 2010-2011 Nokia
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.github.restdriver.clientdriver;

import com.github.restdriver.clientdriver.jetty.DefaultClientDriverJettyHandler;

/**
 * Main entry point to the Rest Client Driver. Just call
 * <code>new {@link ClientDriverFactory}().{@link #createClientDriver()}</code> to get a {@link ClientDriver} running on a
 * free port. You cannot specify which port, one will be chosen for you.
 */
public final class ClientDriverFactory {

    /**
     * Factory method to create and start a {@link ClientDriver}.
     * 
     * @return A new {@link ClientDriver}, which has found a free port, bound to it and started up.
     */
    public ClientDriver createClientDriver() {

        return new ClientDriver(new DefaultClientDriverJettyHandler(new DefaultRequestMatcher()));

    }

    /**
     * Factory method to create and start a {@link ClientDriver} on a specific port.  This is <em>absolutely</em> not the recommended
     * way to use the client driver.  The no-arg method will choose a free port, use of this method will fail if the port is not free.
     *
     * @param port The port to listen on.  If this port is not available a runtime exception will be thrown.
     *
     * @return A new {@link ClientDriver}, which has found a free port, bound to it and started up.
     */
    public ClientDriver createClientDriver(int port) {

        return new ClientDriver(new DefaultClientDriverJettyHandler(new DefaultRequestMatcher()), port);

    }


}
